using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Threading;
using System.Windows.Forms;
using Flip;
using Plux;

[Extension("Flip", OnCreated="OnCreated")]
[Plug("Startup")]
[ParamValue("ExecuteInMainThread", true)]
[Slot("Flip.Front", OnPlugged="OnFrontPlugged", OnUnplugged="OnUnplugged")]
[Slot("Flip.Back", OnPlugged="OnBackPlugged", OnUnplugged="OnUnplugged")]
public class FlipForm : Form, IStartup, IFlipForm, IFlipFormChild {
  readonly IFlipFormChild emptyBackForm;
  readonly IFlipFormChild emptyFrontForm;
  bool _animating;
  Bitmap _backgroundImg;
  Bitmap _backImg;
  bool _flipped;
  Bitmap _frontImg;
  DateTime _lastTime;
  float _stepX, _stepY;
  float _x1, _x2, _y1;
  ExtensionInfo backExtensionInfo;
  double backOpacityBackup;
  bool backShowInTaskbarBackup;
  private bool backKeyPreviewBackup;
  IFlipFormChild currentBack;
  IFlipFormChild currentFront;
  ExtensionInfo frontExtensionInfo;
  double frontOpacityBackup;
  bool frontShowInTaskbarBackup;
  private bool frontKeyPreviewBackup;
  ExtensionInfo me;
  public FlipForm() {
    DoubleBuffered = true;
    Opacity = 0;
    FormBorderStyle = FormBorderStyle.None;
    ShowInTaskbar = false;
    MaximizeBox = false;
    MinimizeBox = false;
    ControlBox = false;
    KeyDown += FlipForm_KeyDown;

    emptyFrontForm = new EmptyFlipForm(true);
    emptyBackForm = new EmptyFlipForm(false);

    emptyFrontForm.Form.Text = "Flip Front";
    emptyFrontForm.FlipFormParent = this;
    //currentFront = emptyFrontForm;

    emptyBackForm.Form.Text = "Flip Back";
    emptyBackForm.FlipFormParent = this;
    currentBack = emptyBackForm;
  }
  bool IsFrontVisible {
    get {
      if(currentFront == null) return true;
      return
        (currentFront == emptyFrontForm)
        ||
        (frontExtensionInfo != null && currentFront == frontExtensionInfo.Object);
    }
  }
  protected override CreateParams CreateParams {
    get {
      CreateParams cp = base.CreateParams;
      // turn on WS_EX_TOOLWINDOW style bit
      cp.ExStyle |= 0x80;
      return cp;
    }
  }
  #region IFlipForm Members
  public void Flip() {
    Flip(2.0f, 0.2f);
  }
  public void FlipSlow() {
    Flip(0.5f, 0.2f);
  }
  public void Flip(float step, float ratio) {
    //if (_animating || frontExtensionInfo == null || backExtensionInfo == null) return;
    if(_animating) return;

    _stepX = step;
    _stepY = step*ratio;

    Size = currentFront.Form.Size;
    Location = currentFront.Form.Location;
    currentBack.Form.Size = Size;
    currentBack.Form.Location = Location;
    _flipped = false;
    _x1 = 0;
    _x2 = Width;
    _y1 = 0;

    _frontImg = new Bitmap(Width, Height);
    _backImg = new Bitmap(Width, Height);
    currentFront.Form.DrawToBitmap(_frontImg, new Rectangle(0, 0, Width, Height));
    currentBack.Form.DrawToBitmap(_backImg, new Rectangle(0, 0, Width, Height));
    //_frontImg.MakeTransparent(Color.Black);
    //_backImg.MakeTransparent(Color.Black);
    _backImg.RotateFlip(RotateFlipType.RotateNoneFlipX);

    frontOpacityBackup = currentFront.Form.Opacity;
    currentFront.Form.Opacity = 0;

    currentFront.Form.Hide();

    _backgroundImg = new Bitmap(Width, Height);
    Thread.Sleep(0);
    using(Graphics gfx = Graphics.FromImage(_backgroundImg))
      gfx.CopyFromScreen(Location.X, Location.Y, 0, 0, _backgroundImg.Size);

    Opacity = frontOpacityBackup;
    _animating = true;
    _lastTime = DateTime.Now;
    BringToFront();
    Invalidate(false);
  }
  #endregion
  #region IFlipFormChild Members
  public Form Form {
    get { return this; }
  }
  public IFlipForm FlipFormParent {
    set { }
  }
  #endregion
  #region IStartup Members
  public void Run() {}
  #endregion
  public void OnCreated(object sender, ExtensionEventArgs args) {
    me = args.ExtensionInfo;
    Runtime.TaskQueue.QueueEmptied += TaskQueue_QueueEmptied;
  }
  void TaskQueue_QueueEmptied(object sender, TaskEventArgs args) {
    if(me.SlotInfos[0].PluggedPlugInfos.Count == 0) {
      frontOpacityBackup = emptyFrontForm.Form.Opacity;
      frontShowInTaskbarBackup = emptyFrontForm.Form.ShowInTaskbar;
      SetForm(emptyFrontForm, true);
    }
    if(me.SlotInfos[1].PluggedPlugInfos.Count == 0) {
      backOpacityBackup = emptyBackForm.Form.Opacity;
      backShowInTaskbarBackup = emptyBackForm.Form.ShowInTaskbar;
      SetForm(emptyBackForm, false);
    }
    Runtime.TaskQueue.QueueEmptied -= TaskQueue_QueueEmptied;
  }
  protected override void OnClosed(EventArgs e) {
    base.OnClosed(e);
    Runtime.Shutdown();
  }
  
  void SetForm(IFlipFormChild flipForm, bool visible) {
    flipForm.FlipFormParent = this;
    flipForm.Form.KeyPreview = true;
    flipForm.Form.KeyDown += Form_KeyDown;
    if(visible) {
      double opacityBackup = flipForm.Form.Opacity;
      flipForm.Form.Opacity = 0;
      flipForm.Form.Show();
      flipForm.Form.ShowInTaskbar = true;
      if(currentFront != null)
        flipForm.Form.Location = currentFront.Form.Location;
      flipForm.Form.Opacity = opacityBackup;

      if(currentFront != null) {
        if (currentFront != flipForm)
          currentFront.Form.Hide();
        currentFront.Form.KeyDown -= Form_KeyDown;
      }
      currentFront = flipForm;
      currentFront.Form.Activate();

      Show();
    }
    else {
      flipForm.Form.Opacity = 0;
      flipForm.Form.Show();
      flipForm.Form.ShowInTaskbar = false;
      if (currentBack != null)
        currentBack.Form.KeyDown -= Form_KeyDown;
      currentBack = flipForm;
    }
  }

  void Form_KeyDown(object sender, KeyEventArgs e) {
    if (!e.Control || ! e.Alt || e.KeyCode != Keys.F) return;
    if (!e.Shift)
      Flip();
    else
      FlipSlow();
  }


  public void OnFrontPlugged(object source, PlugEventArgs args) {
    if(frontExtensionInfo != null) {
      args.PlugInfo.ExtensionInfo.LogEvent(
        new LogEventArgs(MessageType.Warning, VerbosityLevel.Diagnostic,
                         "Front form has already been set!"));
      return;
    }

    frontExtensionInfo = args.PlugInfo.ExtensionInfo;
    IFlipFormChild flipForm = frontExtensionInfo.Object as IFlipFormChild;
    frontOpacityBackup = flipForm.Form.Opacity;
    frontShowInTaskbarBackup = flipForm.Form.ShowInTaskbar;
    frontKeyPreviewBackup = flipForm.Form.KeyPreview;

    if(IsFrontVisible) //{
      SetForm(flipForm, true);
    else
      SetForm(flipForm, false);
  }
  public void OnBackPlugged(object source, PlugEventArgs args) {
    if(backExtensionInfo != null) {
      args.PlugInfo.ExtensionInfo.LogEvent(
        new LogEventArgs(MessageType.Warning, VerbosityLevel.Diagnostic,
                         "Back form has already been set!"));
      return;
    }

    backExtensionInfo = args.PlugInfo.ExtensionInfo;
    IFlipFormChild flipForm = backExtensionInfo.Object as IFlipFormChild;
    backOpacityBackup = flipForm.Form.Opacity;
    backShowInTaskbarBackup = flipForm.Form.ShowInTaskbar;
    backKeyPreviewBackup = flipForm.Form.KeyPreview;

    if(IsFrontVisible) SetForm(flipForm, false);
    else
      SetForm(flipForm, true);

    //flipForm.Form.RightToLeftLayout = true;
    //flipForm.Form.RightToLeft = RightToLeft.Yes;
  }
  public void OnUnplugged(object source, PlugEventArgs args) {
    ExtensionInfo ei = args.PlugInfo.ExtensionInfo;
    IFlipFormChild flipForm = ei.Object as IFlipFormChild;
    if(ei == frontExtensionInfo) {
      if(!flipForm.Form.IsDisposed) {
        try {
          flipForm.Form.ShowInTaskbar = frontShowInTaskbarBackup;
          flipForm.Form.Opacity = frontOpacityBackup;
          flipForm.Form.KeyPreview = frontKeyPreviewBackup;
          flipForm.Form.Hide();
        }
        catch {
          //TODO: FIND ERROR!!!!!
          Console.Write("");
        }
      }
      frontOpacityBackup = 1;
      frontShowInTaskbarBackup = true;
      if(IsFrontVisible) {
        emptyFrontForm.Form.Size = currentFront.Form.Size;
        emptyFrontForm.Form.Location = currentFront.Form.Location;
        emptyFrontForm.Form.Opacity = 1;
        emptyFrontForm.Form.ShowInTaskbar = true;
        SetForm(emptyFrontForm, true);
      }
      else
        SetForm(emptyFrontForm, false);
      frontExtensionInfo = null;
    }
    else if(ei == backExtensionInfo) {
      if(!flipForm.Form.IsDisposed) {
        try {
          flipForm.Form.Opacity = backOpacityBackup;
          flipForm.Form.ShowInTaskbar = backShowInTaskbarBackup;
          flipForm.Form.KeyPreview = backKeyPreviewBackup;
          flipForm.Form.Hide();
        }
        catch {
          Console.WriteLine("");
        }
      }
      backOpacityBackup = 1;
      backShowInTaskbarBackup = true;
      if(IsFrontVisible)
        SetForm(emptyBackForm, false);
      else {
        emptyBackForm.Form.Size = currentFront.Form.Size;
        emptyBackForm.Form.Location = currentFront.Form.Location;
        emptyBackForm.Form.Opacity = 1;
        emptyBackForm.Form.ShowInTaskbar = true;
        SetForm(emptyBackForm, true);
      }
      backExtensionInfo = null;
    }
  }
  // Hide Borderless Form from Alt+Tab
  protected override void OnActivated(EventArgs e) {
    base.OnActivated(e);
    if(currentFront != null)
      currentFront.Form.Activate();
  }
  protected override void OnPaint(PaintEventArgs e) {
    if(_backgroundImg == null)
      return;

    if(_x1 < 0) _x1 = 0;
    if(_x2 > Width) _x2 = Width;
    if(_y1 < 0) _y1 = 0;
    else if(_y1 > Height) _y1 = Height;

    if(!_flipped && _x1 > _x2) {
      IFlipFormChild temp = currentFront;
      currentFront = currentBack;
      currentBack = temp;

      currentFront.Form.Show();
      currentFront.Form.ShowInTaskbar = true;

      double temp2 = frontOpacityBackup;
      frontOpacityBackup = backOpacityBackup;
      backOpacityBackup = temp2;
      Bitmap temp3 = _frontImg;
      _frontImg = _backImg;
      _backImg = temp3;

      _flipped = true;
    }

    int minx = (int) Math.Min(_x1, _x2), maxx = (int) Math.Max(_x1, _x2);
    if(minx != maxx) {
      PointF[] destPoints = {
        new PointF((int) _x1, 0), new Point((int) _x2, (int) _y1),
        new Point((int) _x1, Height - (int) _y1)
      };
      e.Graphics.CompositingMode = CompositingMode.SourceCopy;
      e.Graphics.DrawImage(_frontImg, destPoints);
    }

    DateTime now = DateTime.Now;
    float scale = (float) (now - _lastTime).TotalSeconds;

    _x2 -= _stepX*scale*Width;
    if(_x2 > 0) {
      _x1 += _stepX*scale*Width;
      _y1 += scale*Height*((_x1*2.0f < Width) ? _stepY : -_stepY);
      _lastTime = now;
      Invalidate(new Rectangle(minx, 0, Math.Max(maxx - minx, 1), Height), false);
    }
    else {
      Bitmap temp = _backgroundImg;
      _backgroundImg = null;
      temp.Dispose();
      _frontImg.Dispose();
      _frontImg = null;
      _backImg.Dispose();
      _backImg = null;
      currentFront.Form.Opacity = frontOpacityBackup;
      currentFront.Form.BringToFront();
      currentFront.Form.Activate();
      _animating = false;
      Opacity = 0;
    }
  }
  protected override void OnPaintBackground(PaintEventArgs e) {
    if(_backgroundImg != null) {
      e.Graphics.CompositingMode = CompositingMode.SourceCopy;
      e.Graphics.DrawImageUnscaled(_backgroundImg, new Point(0, 0));
    }
  }
  void FlipForm_KeyDown(object sender, KeyEventArgs e) {
    if(e.KeyCode == Keys.F && e.Modifiers == Keys.Alt) {
      if(e.Modifiers == Keys.Shift)
        FlipSlow();
      else
        Flip();
    }
  }
  protected override void Dispose(bool disposing) {
    base.Dispose(disposing);
    if(emptyFrontForm != null) emptyFrontForm.Form.Dispose();
    if(emptyBackForm != null) emptyBackForm.Form.Dispose();
  }
  void InitializeComponent() {
    SuspendLayout();
    // 
    // FlipForm
    // 
    ClientSize = new Size(292, 266);
    Name = "FlipForm";
    ResumeLayout(false);
  }
}