using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using mRibbon;
using Plux;
using Plux.Framework;


internal class TabInfo {
  public RibbonTab Tab { get; internal set; }
  public PlugTypeHandler ItemHandler { get; internal set; }
}


internal class RibbonButtonEx : RibbonButton {
  public PlugTypeInfo PlugType { get; internal set; }
  public SlotInfo Slot { get; internal set; }
}


[Extension("HotRibbon", Singleton = true)]
[Plug("Plux.ToolStrip", OnPlugged = "Toolbar_Plugged")]
internal class HotRibbon : RibbonControl, IToolStrip {
  private readonly Dictionary<PlugTypeInfo, MenuItemInfo> menuItems
      = new Dictionary<PlugTypeInfo, MenuItemInfo>();
  private Form _mdiContainer;
  private readonly Dictionary<SlotInfo, TabInfo> tabs = new Dictionary<SlotInfo, TabInfo>();
  private Image _unknownIcon;
  private bool hasWindowItemsCreated = false;

  public HotRibbon() {
    using (Stream unknownIconStream =
        GetType().Assembly.GetManifestResourceStream("unknown.ico")) {
      _unknownIcon = Image.FromStream(unknownIconStream);
    }

    SuspendLayout();
    Dock = DockStyle.Fill;

    //OnPopup += OnPopupClicked;

    Stream popupbtnDefaultStream =
      GetType().Assembly.GetManifestResourceStream(
        "popupbtn_default.png");
    Image popupButtonDefault = Image.FromStream(popupbtnDefaultStream);

    SuspendLayout();    
    
    RibbonTab popup = new RibbonTab();
    popup.Tag = popupButtonDefault;
    Controls.Add(popup);
    ResumeLayout();
  }

  void exitButton_Click(object sender, EventArgs e) {
    Runtime.Shutdown();
  }

  private void OnPopupClicked(object sender) {
    ContextMenu menu = new ContextMenu();
    menu.MenuItems.Add("Exit", exitButton_Click);
    menu.Show(sender as Control, new Point(30, 20));
  }

  #region IControl Members

  public Control Control { get { return this; } }

  public void SetCreateBehavior(SlotInfo slot, PlugTypeHandler itemHandler) {
    if (tabs.ContainsKey(slot)) {
      return;
    }
    var info = new TabInfo {
        Tab = new RibbonTab {Text = "View", UseVisualStyleBackColor = true},
        ItemHandler = itemHandler,
    };
    
    Controls.Add(info.Tab);
    Controls.SetChildIndex(info.Tab, 0);
    tabs.Add(slot, info);

    if(!hasWindowItemsCreated) {
      hasWindowItemsCreated = true;
      CreateWindowItems();
    }

    slot.Registered += Registered;
    slot.Deregistered += Deregistered;
    foreach (var plugType in slot.RegisteredPlugTypeInfos) {
      CreateItem(itemHandler(plugType), slot, plugType);
    }
  }

  public void RemoveCreateBehavior(SlotInfo slot) {
    slot.Registered -= Registered;
    slot.Deregistered -= Deregistered;
    TabInfo info = tabs[slot];
    Controls.Remove(info.Tab);
    tabs.Remove(slot);
  }

  private void Registered(object sender, RegisterEventArgs args) {
    TabInfo tabInfo = tabs[args.SlotInfo];
    CreateItem(tabInfo.ItemHandler(args.PlugTypeInfo), args.SlotInfo, args.PlugTypeInfo);
  }

  private void Deregistered(object sender, RegisterEventArgs args) {
    RemoveItem(args.SlotInfo, args.PlugTypeInfo);
  }

  private void CreateItem(string menuItemString, SlotInfo slot, PlugTypeInfo plugType) {
    var menuInfo = new MenuItemInfo(menuItemString);
    menuItems.Add(plugType, menuInfo);
    Image icon = _unknownIcon;
    if (menuInfo.Image != null) {
      if (menuInfo.Image.Length > 0) {
        Type t = plugType.ExtensionTypeInfo.Type;
        using (Stream iconStream = t.Assembly.GetManifestResourceStream(menuInfo.Image)) {
          try {
            icon = new Icon(iconStream, 64, 64).ToBitmap();
          } catch {
            Runtime.GetExtensionInfo(this).LogEvent(
                new LogEventArgs(
                    MessageType.Error, VerbosityLevel.Normal,
                    "Couldn't load icon for action {0}", menuInfo.Name));
          }
        }
      }
    }

    TabInfo tabInfo = tabs[slot];
    RibbonGroup group = null;
    if (!tabInfo.Tab.Controls.ContainsKey(GetGroupName(menuInfo))) {
      group = new RibbonGroup {
          Dock = DockStyle.Left,
          AutoSize = true,
          Name = GetGroupName(menuInfo)
      };
      //group.Text = group.Name;
      //group.Padding = new Padding(2, 0, 3, 18);
      tabInfo.Tab.Controls.Add(group);
    } else {
      group = tabInfo.Tab.Controls[GetGroupName(menuInfo)] as RibbonGroup;
    }
    if (group != null) {
      var btn = new RibbonButtonEx {
          AutoSize = true,
          Dock = DockStyle.Right,
          Name = plugType.ExtensionTypeInfo.Name,
          Text = menuInfo.Name,
          Image = icon,
          TextAlign = ContentAlignment.BottomCenter,
          PlugType = plugType,
          Slot = slot,
      };
      btn.Click += Button_Clicked;
      group.Controls.Add(btn);
    }
  }

  private void RemoveItem(SlotInfo slot, PlugTypeInfo plugType) {
    MenuItemInfo menuItem = menuItems[plugType];
    menuItems.Remove(plugType);
    TabInfo tabInfo = tabs[slot];
    RibbonGroup group = (RibbonGroup)
        tabInfo.Tab.Controls[GetGroupName(menuItem)];
    group.Controls.RemoveByKey(plugType.ExtensionTypeInfo.Name);
    if (group.Controls.Count == 0) {
      tabInfo.Tab.Controls.Remove(group);
    }
  }

  #endregion

  public void Toolbar_Plugged(object s, PlugEventArgs args) {
    Form mdiContainer = (Form)args.SlotInfo.ExtensionInfo.Object;
    _mdiContainer = mdiContainer;
  }

  void CreateWindowItems() {
    RibbonTab windowTab = new RibbonTab {
        Text = "Windows", UseVisualStyleBackColor = true
    };

    RibbonGroup group = new RibbonGroup {
        Dock = DockStyle.Left, 
        AutoSize = true, 
        Name = "Actions", 
        //Padding = new Padding(2, 0, 3, 18),
        //group.Text = "Actions",
    };

    group.Controls.Add(
        CreateWindowButton(
            "Cascade", "window2.ico",
            delegate(object sender, EventArgs e)
            {
              if (_mdiContainer != null) {
                _mdiContainer.LayoutMdi(
                    MdiLayout.Cascade);
              }
            }));

    group.Controls.Add(
        CreateWindowButton(
            "Tile horizontally", "window3.ico",
            delegate(object sender, EventArgs e)
            {
              if (_mdiContainer != null) {
                _mdiContainer.LayoutMdi(MdiLayout.TileHorizontal);
              }
            }));

    group.Controls.Add(
        CreateWindowButton(
            "Tile vertically", "window3.ico",
            delegate(object sender, EventArgs e)
            {
              if (_mdiContainer != null) {
                _mdiContainer.LayoutMdi(MdiLayout.TileVertical);
              }
            }));

    group.Controls.Add(
        CreateWindowButton(
            "Minimize all", "window1.ico",
            delegate(object sender, EventArgs e)
            {
              if (_mdiContainer != null) {
                foreach (Form f in _mdiContainer.MdiChildren) {
                  if (f.MinimizeBox) {
                    f.WindowState = FormWindowState.Minimized;
                  }
                }
                _mdiContainer.LayoutMdi(MdiLayout.ArrangeIcons);
              }
            }));

    group.Controls.Add(
        CreateWindowButton(
            "Restore all", "window1.ico",
            delegate(object sender, EventArgs e)
            {
              if (_mdiContainer != null) {
                foreach (Form f in _mdiContainer.MdiChildren) {
                  f.WindowState = FormWindowState.Normal;
                }
              }
            }));

    windowTab.Controls.Add(group);
    Controls.Add(windowTab);
  }

  public void OnReleased() {
    Hide();
  }

  private RibbonButton CreateWindowButton
      (string text, string icon, EventHandler onClick) {
    RibbonButton btn = new RibbonButton();
    btn.AutoSize = true;
    btn.Dock = DockStyle.Right;
    btn.Text = text;
    btn.TextAlign = ContentAlignment.BottomCenter;
    btn.Click += onClick;

    Stream iconStream = GetType().Assembly.GetManifestResourceStream(icon);
    try {
      btn.Image = new Icon(iconStream, 64, 64).ToBitmap();
    } finally {
      iconStream.Dispose();
    }
    return btn;
  }

  private static string GetGroupName(MenuItemInfo menuItem) {
    string groupName = menuItem.Group ?? "Shortcuts";
    int pos = groupName.IndexOf('&');
    if (pos > -1) {
      groupName = groupName.Remove(pos, 1);
    }
    return groupName;
  }

  private static void Button_Clicked(object sender, EventArgs e) {
    var btn = (RibbonButtonEx)sender;
    btn.Slot.Plug(btn.PlugType);
  }
}


[Extension]
[Plug("Plux.Action")]
[ParamValue("MenuItem", "&View|0.4~&Toolbar|0.04")]
[ParamValue("LazyLoad", false)]
public class ViewToolbar : IAction {
  #region IAction Members

  public void Do(object sender, ActionEventArgs args) {
    // if plugged - unplug
    SlotInfo toolbarSlot =
        args.SlotInfo.ExtensionInfo.FindProvidedSlotTransitive("Plux.ToolStrip");
    if (toolbarSlot.PluggedPlugInfos.Count > 0) {
      toolbarSlot.PluggedPlugInfos[0].Unplug(toolbarSlot);
      return;
    }
    // if not plugged - plug
    PlugTypeInfo plugType = toolbarSlot.RegisteredPlugTypeInfos[0];
    plugType.CreateExtension(toolbarSlot).Plug(toolbarSlot);
  }

  public bool IsEnabled(object sender, ActionEventArgs args) {
    ToolStripMenuItem item = (ToolStripMenuItem)sender;
    SlotInfo toolbarSlot =
        args.SlotInfo.ExtensionInfo.FindProvidedSlotTransitive("Plux.ToolStrip");
    item.Checked =
        toolbarSlot.PluggedPlugInfos.Count > 0;
    return true;
  }

  #endregion
}