using System;
using System.Drawing;
using System.Windows.Forms;
using Plux;
using System.Drawing.Drawing2D;
using System.Text;

namespace Monitor
{
  public partial class SlotButton : UserControl
  {
    public SlotButton()
    {
      InitializeComponent();

      this.SetStyle(ControlStyles.UserPaint, true);
      this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
      this.SetStyle(ControlStyles.DoubleBuffer, true);
      this.SetStyle(ControlStyles.ResizeRedraw, true);
    }

    public SlotButton(ExtensionInfo extension)
    {
      this.extension = extension;

      InitializeComponent();

      ExtensionTypeInfo extTypeInfo = extension.ExtensionTypeInfo;
      txtExtensionName.Text = extension.Name;
      txtExtensionName.BackColor = Color.Transparent;

      txtPluginName.Text = String.Format("{0}, Version {1}",
        extTypeInfo.PluginInfo.Name, extTypeInfo.PluginInfo.Version);
      txtPluginName.BackColor = Color.Transparent;

      SetOpenedSlots();


      if (extension.Name != "Runtime") {
        releaseLink.LinkColor = forecolors[cnt % forecolors.Length];
        releaseLink.BackColor = Color.Transparent;
      } else {
        shutDownLink.LinkColor = forecolors[cnt % forecolors.Length];
        shutDownLink.BackColor = Color.Transparent;
        releaseLink.Visible = false;
        shutDownLink.Visible = true;
      }

      txtSlots.BackColor = forecolors[cnt % forecolors.Length];
      txtSlots.ForeColor = colors[cnt % colors.Length];

      BackColor = colors[cnt++ % colors.Length];
      InitBackground();
    }

    private void releaseLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e) {
      extension.Release();
    }

    private void shutdownLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e) {
      Runtime.Shutdown();
    }

    public void AddRef()
    {
      _refCount++;
      SetOpenedSlots();
    }

    public bool Release()
    {
      if (--_refCount == 0)
        return true;

      SetOpenedSlots();
      return false;
    }

    private void SetOpenedSlots()
    {
      StringBuilder sb = new StringBuilder();
      foreach (SlotInfo slotInfo in extension.SlotInfos)
        if (slotInfo.IsOpen) {
          if (sb.Length > 0) sb.Append(',');
          sb.Append(slotInfo.Name);
        }
      //foreach (string s in _repository.GetOpenedSlots(_instance.ExtensionInfo.Name))
      //{
      //  if (sb.Length > 0) sb.Append(',');
      //  sb.Append(s.Substring(s.IndexOf('.') + 1));
      //}

      txtSlots.Text = sb.ToString();
      txtSlots.Visible = sb.Length > 0;
    }

    private ExtensionInfo extension;
    private int _refCount = 1;

    private static Color[] colors = { Color.Orange, Color.LightSteelBlue, Color.YellowGreen, Color.Yellow, Color.Red, Color.Silver };
    private static Color[] forecolors = { Color.DarkRed, Color.SteelBlue, Color.GreenYellow, Color.SandyBrown, Color.DarkRed, Color.DarkSlateGray };
    private static int cnt = 0;

    public static Color HSL2RGB(double hue, double saturation, double luminance)
    {
      double r = 0.0, g = 0.0, b = 0.0;

      if (luminance == 0.0)
        r = g = b = 0.0;
      else if (saturation == 0.0)
        r = g = b = luminance;
      else
      {
        double aux2 = ((luminance <= 0.5) ? luminance * (1.0 + saturation) : luminance + saturation - (luminance * saturation));
        double aux1 = 2.0 * luminance - aux2;

        double[] t3 = new double[] { hue + 1.0 / 3.0, hue, hue - 1.0 / 3.0 };
        double[] clr = new double[] { 0.0, 0.0, 0.0 };

        for (int i = 0; i < 3; i++)
        {
          if (t3[i] < 0)
            t3[i] += 1.0;
          if (t3[i] > 1)
            t3[i] -= 1.0;

          if (6.0 * t3[i] < 1.0)
            clr[i] = aux1 + (aux2 - aux1) * t3[i] * 6.0;
          else if (2.0 * t3[i] < 1.0)
            clr[i] = aux2;
          else if (3.0 * t3[i] < 2.0)
            clr[i] = (aux1 + (aux2 - aux1) * ((2.0 / 3.0) - t3[i]) * 6.0);
          else
            clr[i] = aux1;
        }

        r = clr[0];
        g = clr[1];
        b = clr[2];
      }

      if (r > 1.0)
        r = 1.0;
      if (g > 1.0)
        g = 1.0;
      if (b > 1.0)
        b = 1.0;

      return Color.FromArgb((int)(255.0 * r), (int)(255.0 * g), (int)(255.0 * b));
    }

    private Color GetColor(double luminance)
    {
      Color col = HSL2RGB(BackColor.GetHue() / 360.0,
          BackColor.GetSaturation() / 1.5,
          BackColor.GetBrightness() * 1.025);

      double r = (double)col.R * luminance;
      double g = (double)col.G * luminance;
      double b = (double)col.B * luminance;

      if (r > 255.0)
        r = 255.0;
      if (g > 255.0)
        g = 255.0;
      if (b > 255.0)
        b = 255.0;

      return Color.FromArgb((int)r, (int)g, (int)b);
    }

    private void InitBackground()
    {
      const double d = 0.075;
      _blend = new ColorBlend();
      _blend.Colors = new Color[] { GetColor(1.0 + d), GetColor(1.0), GetColor(1.0 - d), GetColor(1.0 + 3 * d) };
      _blend.Positions = new float[] { 0.0f, 0.2f, 0.2f, 1.0f };
    }

    protected override void OnPaintBackground(PaintEventArgs e)
    {
      LinearGradientBrush brush = new LinearGradientBrush(DisplayRectangle, Color.Transparent, Color.Transparent, LinearGradientMode.Vertical);
      brush.InterpolationColors = _blend;
      e.Graphics.FillRectangle(brush, e.ClipRectangle);
    }

    private ColorBlend _blend;
  }
}
