using System;
using System.IO;

namespace Plux.Framework {

  public class ScriptReader {
    private readonly IScriptExecutor executor;
    private readonly ExtensionInfo ext;
    public ScriptReader(IScriptExecutor executor, ExtensionInfo ext) {
      this.executor = executor;
      this.ext = ext;
    }
    public void ExecuteScript(string script) {
      using(
        ResourceStreamGetter rsg =
          new ResourceStreamGetter(ext.ExtensionTypeInfo.PluginInfo, script, false)) {
        string s;
        using(StreamReader r = new StreamReader(rsg.Stream))
          s = r.ReadToEnd();
        executor.ExecuteAsync(s, ScriptCompleted);
      }
    }
    private void ScriptCompleted(object s, ScriptCompletedEventArgs args) {
      ext.LogEvent(args.Result.StateMessage);
    }

    public static void ExecuteScript(string scriptPath, ExtensionInfo ext) {

      if (string.IsNullOrEmpty(scriptPath)) {
        ext.LogEvent(new LogEventArgs(MessageType.Error, VerbosityLevel.Minimal, "script path is empty."));
        return;
      }
      if (!File.Exists(scriptPath)) {
        ext.LogEvent(new LogEventArgs(MessageType.Error, VerbosityLevel.Minimal, "script file not found: \"{0}\"", scriptPath));
        return;
      }

      try {
        using (StreamReader sr = new StreamReader(scriptPath)) {
          String line;
          CommandInterpreter cmdInterpreter = new CommandInterpreter();
          RuntimeCommands runtimeCommands = new RuntimeCommands();
          foreach (ICommand cmd in runtimeCommands.Commands) {
            cmdInterpreter.commands.Add(cmd.Name, cmd);
          }
          ext.LogEvent(new LogEventArgs(MessageType.Info, VerbosityLevel.Normal, "script \"{0}\" started", scriptPath));
          while ((line = sr.ReadLine()) != null) {
            string result = cmdInterpreter.Execute(line);
            string log = String.Format("command: \"{0}\"", line);
            if (!String.IsNullOrEmpty(result)) log = String.Format("{0}{1}  {2}", log, Environment.NewLine, result);
            ext.LogEvent(new LogEventArgs(MessageType.Info, VerbosityLevel.Diagnostic, log));
          }
          ext.LogEvent(new LogEventArgs(MessageType.Info, VerbosityLevel.Normal, "script \"{0}\" finished", scriptPath));
        }
      } catch(Exception e) {
        ext.LogEvent(new LogEventArgs(
          MessageType.Error, VerbosityLevel.Minimal, "error on reading script file: \"{0}\"", scriptPath));
        ext.LogEvent(new LogEventArgs(MessageType.Error, VerbosityLevel.Minimal, e.ToString()));
      }
    }

      /*
    public static void ExecutePSScript(string scriptPath, ExtensionInfo ext)
    {

        if (string.IsNullOrEmpty(scriptPath))
        {
            ext.LogEvent(new LogEventArgs(MessageType.Error, VerbosityLevel.Minimal, "script path is empty."));
            return;
        }
        if (!File.Exists(scriptPath))
        {
            ext.LogEvent(new LogEventArgs(MessageType.Error, VerbosityLevel.Minimal, "script file not found: \"{0}\"", scriptPath));
            return;
        }

        try
        {
            using (StreamReader sr = new StreamReader(scriptPath))
            {
                ext.LogEvent(new LogEventArgs(MessageType.Info, VerbosityLevel.Normal, "script \"{0}\" started", scriptPath));
                string scriptBody = sr.ReadToEnd();

                string result = ZCommandInterpreter.Execute(scriptBody);
                string log = String.Format("Script body: \"{0}\"", scriptBody);
                if (!String.IsNullOrEmpty(result)) log = String.Format("{0}{1}{1}Result:{1}{2}", log, Environment.NewLine, result);
                ext.LogEvent(new LogEventArgs(MessageType.Info, VerbosityLevel.Diagnostic, log));
                ext.LogEvent(new LogEventArgs(MessageType.Info, VerbosityLevel.Normal, "script \"{0}\" finished", scriptPath));
            }
        }
        catch (CommandExecutionException e)
        {
            ext.LogEvent(new LogEventArgs(
              MessageType.Error, VerbosityLevel.Minimal, "error on reading script file: \"{0}\"", scriptPath));
            ext.LogEvent(new LogEventArgs(MessageType.Error, VerbosityLevel.Minimal, e.ToString()));
        }
    }
       * */
  
  }
}
