using System;
using System.Collections.Generic;

namespace Plux.Framework {
  [Extension("Plux.Framework")]
  [Plug("Startup")]
  [ParamValue("ExecuteInMainThread", true)]
  internal class Startup : IStartup {
    #region IStartup Members
    public void Run() {
      SlotOpenHistory.Init();
      PreferencesStartup.Register();
      new ReplaceRegisterStartup().Init();
      new ReplacePlugStartup().Init();
      new DependentPlugStartup().Init();
      new QueuePlugsStartup().Init();

      ExtensionInfo ext = Runtime.GetExtensionInfo(this);

      if (Runtime.Arguments.ContainsKey("script"))
        ScriptReader.ExecuteScript(Runtime.Arguments["script"], ext);
      ext.Release();
    }
    #endregion
  }

  public static class PreferencesStartup {
    static bool done = false;
    public static void Register() {
      Runtime.Repository.PluginAdded += PluginAdded;
      foreach (SlotTypeInfo slotType in Runtime.Repository.SlotTypeInfos) {
        if (!slotType.Name.Equals("Plux.Preferences")) continue;
        slotType.Registered += Persistor_Registered;
      }
    }
    static void PluginAdded(object sender, PluginEventArgs args) {
      if (done) Runtime.Repository.PluginAdded -= PluginAdded;
      foreach (ExtensionTypeInfo type in args.ExtensionTypeInfos)
        foreach (SlotTypeInfo slotType in type.SlotTypeInfos) {
          if (!slotType.Name.Equals("Plux.Preferences")) continue;
          slotType.Registered += Persistor_Registered;
        }
    }
    static void Persistor_Registered(object sender, RegisterEventArgs args) {
      if (!done) {
        args.PlugTypeInfo.ExtensionTypeInfo.GetSharedExtension(true);
        done = true;
      }
      args.SlotInfo.SlotTypeInfo.Registered -= Persistor_Registered;
    }
  }
  public abstract class FrameworkPlugAttribute : Attribute {
    internal readonly string Name;
    protected FrameworkPlugAttribute(string name) {
      Name = name;
    }
  }

  public abstract class FrameworkSlotAttribute : Attribute {
    internal readonly string Name;
    protected FrameworkSlotAttribute(string name) {
      Name = name;
    }
  }

  public abstract class FrameworkStartup {
    public virtual void Init() {
      Runtime.Repository.PluginAdded += PluginAdded;
      Runtime.Repository.PluginRemoved += PluginRemoved;
      foreach(PluginInfo pi in Runtime.Repository.PluginInfos) Register(pi);
    }
    private void PluginAdded(object sender, PluginEventArgs args) {
      Register(args.PluginInfo);
    }
    private void PluginRemoved(object sender, PluginEventArgs args) {
      Unregister(args.PluginInfo);
    }
    protected abstract void Register(PluginInfo pi);
    protected abstract void Unregister(PluginInfo pi);
  }

  public abstract class FrameworkStartup<T> : FrameworkStartup where T : Attribute {
    protected override void Register(PluginInfo pi) {
      foreach(ExtensionTypeInfo typeInfo in pi.ExtensionTypeInfos) {
        IList<T> attr = AssemblyAnalyzer.GetCustomAttributes<T>(typeInfo.Type);
        if(attr.Count > 0) {
          typeInfo.ExtensionCreated += ExtensionCreated;
          typeInfo.ExtensionReleased += ExtensionReleased;
          Register(typeInfo);
        }
      }
    }
    protected override void Unregister(PluginInfo pi) {
      foreach(ExtensionTypeInfo typeInfo in pi.ExtensionTypeInfos) {
        typeInfo.ExtensionCreated -= ExtensionCreated;
        Unregister(typeInfo);
      }
    }
    public abstract void Register(ExtensionTypeInfo extType);
    public abstract void Unregister(ExtensionTypeInfo extType);
    public abstract void ExtensionCreated(object s, ExtensionEventArgs args);
    public abstract void ExtensionReleased(object s, ExtensionEventArgs args);
  }

  public abstract class PlugStartup<T>
    : FrameworkStartup<FrameworkPlugAttribute> where T : FrameworkPlugAttribute {
    public override void ExtensionCreated(object s, ExtensionEventArgs args) {

      IList<T> attr = AssemblyAnalyzer.GetCustomAttributes<T>(args.ExtensionTypeInfo.Type);
      foreach(T t in attr) PlugCreated(args.ExtensionInfo.PlugInfos[t.Name], t);
    }
    public abstract void PlugCreated(PlugInfo plug, T attr);
    public override void ExtensionReleased(object s, ExtensionEventArgs args) {
      IList<T> attr =
        AssemblyAnalyzer.GetCustomAttributes<T>(args.ExtensionTypeInfo.Type);
      foreach(T t in attr)
        PlugReleased(args.ExtensionInfo.PlugInfos[t.Name], t);
    }
    public abstract void PlugReleased(PlugInfo plug, T attr);
    }

  internal abstract class SlotStartup<T> :
    FrameworkStartup<T>
    where T : FrameworkSlotAttribute {
    public override void ExtensionCreated(object s, ExtensionEventArgs args) {
      IList<T> attr = AssemblyAnalyzer.GetCustomAttributes<T>(args.ExtensionTypeInfo.Type);
      //IList<T> attr = args.ExtensionTypeInfo.GetCustomAttributes<T>();
      foreach(T t in attr) SlotCreated(args.ExtensionInfo.SlotInfos[t.Name], t);
    }
    public abstract void SlotCreated(SlotInfo slot, T attr);
    public override void ExtensionReleased(object s, ExtensionEventArgs args) {
      IList<T> attr =
        AssemblyAnalyzer.GetCustomAttributes<T>(args.ExtensionTypeInfo.Type);
      foreach(T t in attr)
        SlotReleased(args.ExtensionInfo.SlotInfos[t.Name], t);
    }
    public abstract void SlotReleased(SlotInfo slot, T attr);
    }
}