﻿using System;
using System.Drawing;
using System.Windows.Forms;
using Plux;
using Plux.Framework;
using System.Linq;


internal class MdiManager {
  private bool isMaximized = false;
  private bool noUpdate = false;
  private SlotInfo viewSlot;
  private Form mainForm;
  private Icon defaultIcon;

  public MdiManager(SlotInfo viewSlot, Form parent, Icon defaultIcon) {
    this.viewSlot = viewSlot;
    mainForm = parent;
    this.defaultIcon = defaultIcon;
    viewSlot.SelectionChanged += OnSelectionChanged;
  }

  public void Form_Activated(object sender, EventArgs args) {
    if (noUpdate || !Runtime.IsCompositionAllowed) {
      return;
    }
    noUpdate = true;
    viewSlot.SelectedPlug = (PlugInfo) ((Form) sender).Tag;
    noUpdate = false;
  }

  public void Form_Closed(object sender, EventArgs args) {
    if (noUpdate) {
      return;
    }
    var view = (PlugInfo) ((Form) sender).Tag;
    if (view.ExtensionInfo.IsReleased) {
      return;
    }
    noUpdate = true;
    viewSlot.Unplug(view);
    noUpdate = false;
  }

  private void Form_LocationChanged(object sender, EventArgs args) {
    Form form = (Form) sender;
    isMaximized = form.WindowState == FormWindowState.Maximized;
  }

  private void OnSelectionChanged(object sender, SelectionChangedEventArgs args) {
    if (noUpdate) {
      return;
    }
    if (args.AddedPlugs.Count == 0) {
      return;
    }
    BringToFront(args.AddedPlugs[0]);
  }

  public Form CreateChildForm(PlugInfo plug) {
    IView view = (IView) plug.Object;
    view.Control.Dock = DockStyle.Fill;

    var form = new Form {
        Icon = GetIcon(plug),
        Tag = plug,
        StartPosition = FormStartPosition.WindowsDefaultBounds,
        Text = view.Name,
        MdiParent = mainForm,
    };
    form.LocationChanged += Form_LocationChanged;
    form.Activated += Form_Activated;
    form.Closed += Form_Closed;
    form.Controls.Add(view.Control);
    if (isMaximized && viewSlot.PluggedPlugInfos.Count == 1) {
      form.WindowState = FormWindowState.Maximized;
    }
    noUpdate = true; // OnActivate+OnSelectionChanged
    form.Show();
    noUpdate = false;
    Form_Activated(form, null); // set selection on slot
    return form;
  }

  private Form GetChildForm(PlugInfo plug) {
    foreach (Form f in mainForm.MdiChildren) {
      if (f.Tag == plug) {
        return f;
      }
    }
    return null;
  }

  public void CloseChildForm(PlugInfo plug) {
    if (!noUpdate) {
      GetChildForm(plug).Close();
    }
  }

  public void BringToFront(PlugInfo plug) {
    GetChildForm(plug).BringToFront();
  }

  private Icon GetIcon(PlugInfo plug) {
    string icon = (string) plug.PlugTypeInfo.GetParamValue("Icon");
    if (icon.Length > 0) {
      return new Icon(plug.ExtensionInfo.ExtensionTypeInfo.Type, icon);
    }
    return defaultIcon;
  }

  public void SaveSettings(Preferences pref) {
    pref.Set("WindowStateMaximized", isMaximized);
  }

  public void RestoreSettings(Preferences pref) {
    isMaximized = (bool) pref.Get("WindowStateMaximized", false);
  }
}